/* dft.c - discrete fourier transformation program */

/*
 * author: frank.zirkelbach@physik.uni-augsburg.de
 *
 */

#include "dft.h"

int main(int argc,char **argv) {

  t_fourier fourier;
  t_bmp src;
  t_bmp dst;
  t_bmp cut;
  int x,y;
  int offy,offt;
  int scale;
  double max;
  double *mag;

  bmp_init(&src,1);
  bmp_init(&dst,1);
  bmp_init(&cut,1);
  src.mode=READ;
  dst.mode=WRITE;
  cut.mode=WRITE;
  strcpy(src.file,argv[1]);
  strcpy(dst.file,argv[2]);
  strcpy(cut.file,argv[3]);

  fourier_init(&fourier,1);
  fourier.type=DFT|FWD;
  fourier.dim=2;

  bmp_read_file(&src);
  bmp_cut_bottom(&cut,&src,150);
  bmp_write_file(&cut);

  dst.width=src.info.width;
  dst.height=src.info.height;
  bmp_alloc_map(&dst);

  fourier.data_len[0]=src.info.width;
  fourier.data_len[1]=src.info.height;
  fourier_alloc_data(&fourier);

  mag=(double *)malloc(fourier.data_len[0]*fourier.data_len[1]*sizeof(double));
  if(mag==NULL) {
    printf("unable to alloc mag memory\n");
    return -1;
  }

  // copy the data
  offy=0;
  for(y=0;y<fourier.data_len[1];y++) {
    for(x=0;x<fourier.data_len[0];x++) {
      offt=offy+x;
      fourier.data[offt].r=(src.map[offt].r+src.map[offt].g+src.map[offt].b)/3;
    }
    offy+=fourier.data_len[0];
  }

  // do the fourier trafo
  fourier_dft_2d(&fourier);

  printf("fourier done!\n");

  // copy back the data, intensity = sqrt(r^2+i^2)
  max=0;
  offt=0;
  for(y=0;y<fourier.data_len[1];y++) {
    for(x=0;x<fourier.data_len[0];x++) {
      mag[offt]=sqrt(fourier.ftdata[offt].r*fourier.ftdata[offt].r+fourier.ftdata[offt].i*fourier.ftdata[offt].i);
      if(mag[offt]>max) max=mag[offt];
      offt+=1;
    }
  }

  printf("found max: %f\n",max);

  if(max!=0) scale=(int)255/max;
  else scale=0;

  if(scale!=0) {
    printf("scaling image intensity: %d\n",scale);
    offt=0;
    for(y=0;y<dst.height;y++) {
      for(x=0;x<dst.width;x++) {
        offy=((y<dst.height/2)?y+dst.height/2:y-dst.height/2);
        offy*=dst.width;
        offy+=((x<dst.width/2)?x+dst.width/2:x-dst.width/2);
        dst.map[offy].r=scale*mag[offt];
        dst.map[offy].g=dst.map[offy].r;
        dst.map[offy].b=dst.map[offy].r;
        offt+=1;
      }
    }
  }

  free(mag);

  bmp_write_file(&dst);

  fourier_shutdown(&fourier);

  bmp_shutdown(&src);
  bmp_shutdown(&dst);

  printf("done ...\n");

  return 1;
}
